@@ -25,9 +25,10 @@ from baangt.base.PathManagement import ManagedPaths
 from uuid import uuid4
 from baangt.base.FilesOpen import FilesOpen
 from baangt.base.RuntimeStatistics import Statistic
-from baangt.base.PathManagement import ManagedPaths
+#from baangt.base.PathManagement import ManagedPaths
 from baangt.base.DownloadFolderMonitoring import DownloadFolderMonitoring
 from baangt.base.Cleanup import Cleanup
+from baangt.base.ResultsBrowser import ResultsBrowser
 import xlrd3 as xlrd
 from baangt.reports import Dashboard, Summary
 from baangt.TestDataGenerator.TestDataGenerator import TestDataGenerator
@@ -35,6 +36,8 @@ from baangt.base.Utils import utils
 from threading import Thread
 from time import sleep
 import signal
+from datetime import datetime
+
 
 logger = logging.getLogger("pyC")
 
@@ -131,6 +134,13 @@ class MainWindow(Ui_MainWindow):
         self.openTestFilePushButton_4.clicked.connect(self.openTestFile)
         self.InputFileOpen.clicked.connect(self.openInputFile)
 
+        # Result Browser buttons
+        self.actionQuery.triggered.connect(self.showQueryPage)
+        self.queryMakePushButton.clicked.connect(self.makeResultQuery)
+        self.queryExportPushButton.clicked.connect(self.exportResultQuery)
+        self.openExportPushButton.clicked.connect(self.openRecentQueryResults)
+        self.queryExitPushButton.clicked.connect(self.mainPageView)
+
         # Quit Event
         self.actionExit.triggered.connect(self.quitApplication)
 
@@ -239,6 +249,7 @@ class MainWindow(Ui_MainWindow):
         logo_pixmap = QtGui.QPixmap(":/baangt/baangtlogo")
         logo_pixmap.scaled(300, 120, QtCore.Qt.KeepAspectRatio)
         self.logo_4.setPixmap(logo_pixmap)
+        self.queryLogo.setPixmap(logo_pixmap)
         icon = QtGui.QIcon()
         icon.addPixmap(
                 QtGui.QPixmap(":/baangt/baangticon"),
@@ -664,6 +675,8 @@ class MainWindow(Ui_MainWindow):
             if result_file[-5:] == ".xlsx":
                 logger.debug(f"Found result_file in the logs: {result_file}")
                 self.__result_file = result_file
+        elif "Logfile used: " in text:
+            self.__log_file = text.split("used: ")[1].strip()
         if "||Statistic:" in text:
             lis = text.split('||')
             stat = lis[1][10:]
@@ -1334,6 +1347,92 @@ class MainWindow(Ui_MainWindow):
             self.__open_files = 0
         self.saveInteractiveGuiConfig()
 
+
+    #
+    # Browse Results actions
+    #
+
+    @pyqtSlot()
+    def showQueryPage(self):
+        #
+        # display Query Page
+        #
+
+        # setup query object
+        self.queryResults = ResultsBrowser()
+
+        # setup combo box lists
+        self.nameComboBox.clear()
+        self.nameComboBox.addItems([''] + self.queryResults.name_list())
+        self.stageComboBox.clear()
+        self.stageComboBox.addItems([''] + self.queryResults.stage_list())
+
+        # show the page
+        self.stackedWidget.setCurrentIndex(4)
+        self.statusMessage("Query Page is triggered", 1000)
+
+
+    @pyqtSlot()
+    def makeResultQuery(self):
+        #
+        # makes query to results db
+        #
+
+        # get field data
+        name = self.nameComboBox.currentText() or None
+        stage = self.stageComboBox.currentText() or None
+        date_from = datetime.strptime(self.dateFromInput.date().toString("yyyyMMdd"), "%Y%m%d")
+        date_to = datetime.strptime(self.dateToInput.date().toString("yyyyMMdd"), "%Y%m%d")
+
+        # make query
+        self.queryResults.query(name=name, stage=stage, start_date=date_from, end_date=date_to)
+
+        # display status
+        if self.queryResults.query_set:
+            if self.queryResults.query_set.length > 1:
+                status = f'Found: {self.queryResults.query_set.length} records'
+            else:
+                status = f'Found: {self.queryResults.query_set.length} record'
+        else:
+            status = 'No record found'
+        self.queryStatusLabel.setText(QtCore.QCoreApplication.translate("MainWindow", status))
+
+
+    @pyqtSlot()
+    def exportResultQuery(self):
+        #
+        # exports query results
+        #
+
+        _translate = QtCore.QCoreApplication.translate
+
+        if self.queryResults.query_set:
+            #self.queryStatusLabel.setText(_translate("MainWindow", "Exporting results..."))
+            #sleep(1)
+            path_to_export = self.queryResults.export()
+            self.queryStatusLabel.setText(_translate("MainWindow", f"Exported to: {path_to_export}"))
+            FilesOpen.openResultFile(path_to_export)
+        else:
+            self.queryStatusLabel.setText(_translate("MainWindow", f"ERROR: No data to export"))
+
+    @pyqtSlot()
+    def openRecentQueryResults(self):
+        #
+        # opens recent query result file
+        #
+
+        try:
+            # get recent file
+            recent_export = max(
+                list(Path(self.managedPaths.getOrSetDBExportPath()).glob('*.xlsx')),
+                key=os.path.getctime,
+            )
+            FilesOpen.openResultFile(recent_export)
+        except Exception:
+            # no export file exists
+            self.statusMessage(f"No Result Export File to Show", 3000)
+
+
     @pyqtSlot()
     def cleanup_dialog(self):
         self.clean_dialog = QtWidgets.QDialog(self.centralwidget)
@@ -1348,6 +1447,9 @@ class MainWindow(Ui_MainWindow):
         self.cleanup_downloads = QtWidgets.QCheckBox()
         self.cleanup_downloads.setText("Downloads")
         self.cleanup_downloads.setChecked(True)
+        self.cleanup_reports = QtWidgets.QCheckBox() # cleanup reports
+        self.cleanup_reports.setText("Reports") # cleanup reports
+        self.cleanup_reports.setChecked(True) # cleanup reports
         hlay = QtWidgets.QHBoxLayout()
         label = QtWidgets.QLabel()
         label.setText("Days: ")
@@ -1362,6 +1464,7 @@ class MainWindow(Ui_MainWindow):
         vlay.addWidget(self.cleanup_logs)
         vlay.addWidget(self.cleanup_screenshots)
         vlay.addWidget(self.cleanup_downloads)
+        vlay.addWidget(self.cleanup_reports) # cleanup reports
         vlay.addLayout(hlay)
         vlay.addWidget(button)
         vlay.addWidget(self.cleanup_status)
@@ -1384,6 +1487,9 @@ class MainWindow(Ui_MainWindow):
             c.clean_screenshots()
         if self.cleanup_downloads.isChecked():
             c.clean_downloads()
+        # cleanup reports
+        if self.cleanup_reports.isChecked():
+            c.clean_reports()
         self.cleanup_status.showMessage("Cleaning Complete!")
 
     def showReport(self):
